@@ -153,10 +153,10 @@ class Axis(ABCAxis):
 
         Examples
         --------
-        >>> from larray import ndrange
+        >>> from larray import ndtest
         >>> sex = Axis('sex=M,F')
         >>> time = Axis([2007, 2008, 2009, 2010], 'time')
-        >>> arr = ndrange([sex, time])
+        >>> arr = ndtest([sex, time])
         >>> arr
         sex\\time  2007  2008  2009  2010
                M     0     1     2     3
@@ -184,20 +184,7 @@ class Axis(ABCAxis):
             labels = np.arange(length)
             iswildcard = True
         else:
-            # TODO: move this to _to_ticks????
-            # we convert to an ndarray to save memory for scalar ticks (for
-            # LGroup ticks, it does not make a difference since a list of LGroup
-            # and an ndarray of LGroup are both arrays of pointers)
-            ticks = _to_ticks(labels, parse_single_int=True)
-            if _contain_group_ticks(ticks):
-                # avoid getting a 2d array if all LGroup have the same length
-                labels = np.empty(len(ticks), dtype=object)
-                # this does not work if some values have a length (with a valid __len__) and others not
-                # labels[:] = ticks
-                for i, tick in enumerate(ticks):
-                    labels[i] = tick
-            else:
-                labels = np.asarray(ticks)
+            labels = _to_ticks(labels, parse_single_int=True)
             length = len(labels)
             iswildcard = False
 
@@ -1067,12 +1054,21 @@ class Axis(ABCAxis):
 
         Examples
         --------
-        >>> letters = Axis('letters=a,b')
-        >>> letters.union(Axis('letters=b,c'))
-        Axis(['a', 'b', 'c'], 'letters')
-        >>> letters.union(['b', 'c'])
-        Axis(['a', 'b', 'c'], 'letters')
+        >>> a = Axis('a=a0..a2')
+        >>> a.union('a1')
+        Axis(['a0', 'a1', 'a2'], 'a')
+        >>> a.union('a3')
+        Axis(['a0', 'a1', 'a2', 'a3'], 'a')
+        >>> a.union(Axis('a=a1..a3'))
+        Axis(['a0', 'a1', 'a2', 'a3'], 'a')
+        >>> a.union('a1..a3')
+        Axis(['a0', 'a1', 'a2', 'a3'], 'a')
+        >>> a.union(['a1', 'a2', 'a3'])
+        Axis(['a0', 'a1', 'a2', 'a3'], 'a')
         """
+        if isinstance(other, basestring):
+            # TODO : remove [other] if ... when FuturWarning raised in Axis.init will be removed
+            other = _to_ticks(other, parse_single_int=True) if '..' in other or ',' in other else [other]
         if isinstance(other, Axis):
             other = other.labels
         unique_labels = []
@@ -1098,12 +1094,21 @@ class Axis(ABCAxis):
 
         Examples
         --------
-        >>> letters = Axis('letters=a,b')
-        >>> letters.intersection(Axis('letters=b,c'))
-        Axis(['b'], 'letters')
-        >>> letters.intersection(['b', 'c'])
-        Axis(['b'], 'letters')
+        >>> a = Axis('a=a0..a2')
+        >>> a.intersection('a1')
+        Axis(['a1'], 'a')
+        >>> a.intersection('a3')
+        Axis([], 'a')
+        >>> a.intersection(Axis('a=a1..a3'))
+        Axis(['a1', 'a2'], 'a')
+        >>> a.intersection('a1..a3')
+        Axis(['a1', 'a2'], 'a')
+        >>> a.intersection(['a1', 'a2', 'a3'])
+        Axis(['a1', 'a2'], 'a')
         """
+        if isinstance(other, basestring):
+            # TODO : remove [other] if ... when FuturWarning raised in Axis.init will be removed
+            other = _to_ticks(other, parse_single_int=True) if '..' in other or ',' in other else [other]
         if isinstance(other, Axis):
             other = other.labels
         to_keep = set(other)
@@ -1126,12 +1131,21 @@ class Axis(ABCAxis):
 
         Examples
         --------
-        >>> letters = Axis('letters=a,b')
-        >>> letters.difference(Axis('letters=b,c'))
-        Axis(['a'], 'letters')
-        >>> letters.difference(['b', 'c'])
-        Axis(['a'], 'letters')
+        >>> a = Axis('a=a0..a2')
+        >>> a.difference('a1')
+        Axis(['a0', 'a2'], 'a')
+        >>> a.difference('a3')
+        Axis(['a0', 'a1', 'a2'], 'a')
+        >>> a.difference(Axis('a=a1..a3'))
+        Axis(['a0'], 'a')
+        >>> a.difference('a1..a3')
+        Axis(['a0'], 'a')
+        >>> a.difference(['a1', 'a2', 'a3'])
+        Axis(['a0'], 'a')
         """
+        if isinstance(other, basestring):
+            # TODO : remove [other] if ... when FuturWarning raised in Axis.init will be removed
+            other = _to_ticks(other, parse_single_int=True) if '..' in other or ',' in other else [other]
         if isinstance(other, Axis):
             other = other.labels
         to_drop = set(other)
@@ -1411,9 +1425,8 @@ class AxisCollection(object):
             for k, v in zip(key, value):
                 self[k] = v
         else:
-            if isinstance(value, (int, basestring, list, tuple)):
+            if not isinstance(value, Axis):
                 value = Axis(value)
-            assert isinstance(value, Axis)
             idx = self.index(key)
             step = 1 if idx >= 0 else -1
             self[idx:idx + step:step] = [value]
@@ -1900,7 +1913,7 @@ class AxisCollection(object):
 
         Examples
         --------
-        >>> from larray import ndtest, ndrange
+        >>> from larray import ndtest
         >>> axes = ndtest((2, 3)).axes
         >>> axes
         AxisCollection([
@@ -1941,7 +1954,7 @@ class AxisCollection(object):
             Axis(['r0', 'r1'], 'row'),
             Axis(['c0', 'c1', 'c2'], 'column')
         ])
-        >>> arr = ndrange([row, column])
+        >>> arr = ndtest([row, column])
         >>> axes.replace(arr.axes)
         AxisCollection([
             Axis(['r0', 'r1'], 'row'),
