@@ -9,7 +9,7 @@ from collections import OrderedDict
 import numpy as np
 
 from larray.core.axis import Axis
-from larray.core.array import LArray, larray_nan_equal, get_axes, ndtest, zeros, zeros_like, sequence
+from larray.core.array import LArray, get_axes, ndtest, zeros, zeros_like, sequence, aslarray
 from larray.util.misc import float_error_handler_factory, is_interactive_interpreter, renamed_to, inverseop
 from larray.inout.session import check_pattern, handler_classes, ext_default_engine
 
@@ -628,14 +628,17 @@ class Session(object):
         arr2: 4
          a [4]: 'a0' 'a1' 'a2' 'a3'
         dtype: int64
+        memory used: 32 bytes
         arr1: 2 x 2
          a [2]: 'a0' 'a1'
          b [2]: 'b0' 'b1'
         dtype: int64
+        memory used: 32 bytes
         arr3: 3 x 2
          a [3]: 'a0' 'a1' 'a2'
          b [2]: 'b0' 'b1'
         dtype: int64
+        memory used: 48 bytes
         """
         return self._objects.items()
 
@@ -651,21 +654,23 @@ class Session(object):
 
         def opmethod(self, other):
             self_keys = set(self.keys())
-            all_keys = list(self.keys()) + [n for n in other.keys() if n not in self_keys]
+            all_keys = list(self.keys())
+            if hasattr(other, 'keys'):
+                all_keys += [n for n in other.keys() if n not in self_keys]
             with np.errstate(call=_session_float_error_handler):
                 res = []
                 for name in all_keys:
                     self_array = self.get(name, np.nan)
-                    other_array = other.get(name, np.nan)
+                    other_operand = other.get(name, np.nan) if hasattr(other, 'get') else other
                     try:
-                        res_array = getattr(self_array, opfullname)(other_array)
+                        res_array = getattr(self_array, opfullname)(other_operand)
                     # TypeError for str arrays, ValueError for incompatible axes, ...
                     except Exception:
                         res_array = np.nan
                     # this should only ever happen when self_array is a non Array (eg. nan)
                     if res_array is NotImplemented:
                         try:
-                            res_array = getattr(other_array, '__%s__' % inverseop(opname))(self_array)
+                            res_array = getattr(other_operand, '__%s__' % inverseop(opname))(self_array)
                         # TypeError for str arrays, ValueError for incompatible axes, ...
                         except Exception:
                             res_array = np.nan
@@ -675,9 +680,16 @@ class Session(object):
         return opmethod
 
     __add__ = _binop('add')
+    __radd__ = _binop('radd')
     __sub__ = _binop('sub')
+    __rsub__ = _binop('rsub')
     __mul__ = _binop('mul')
+    __rmul__ = _binop('rmul')
     __truediv__ = _binop('truediv')
+    __rtruediv__ = _binop('rtruediv')
+
+    __eq__ = _binop('eq')
+    __ne__ = _binop('ne')
 
     # element-wise method factory
     # unary operations are (also) dispatched element-wise to all arrays
@@ -702,15 +714,108 @@ class Session(object):
     __abs__ = _unaryop('abs')
     __invert__ = _unaryop('invert')
 
-    # XXX: use _binop (ie elementwise comparison instead of aggregating directly?)
-    def __eq__(self, other):
+    def array_equals(self, other):
+        """Test if arrays of the current session are equal to those of another session.
+
+        Equivalent to apply :py:meth:`LArray.equals` with flag nan_equals=True to all arrays from two sessions.
+
+        Parameters
+        ----------
+        other : Session
+            Session to compare with.
+
+        Returns
+        -------
+        Boolean LArray
+
+        See Also
+        --------
+        Session.equals
+
+        Examples
+        --------
+        >>> s1 = Session([('arr1', ndtest(2)), ('arr2', ndtest((2, 2)))])
+        >>> s2 = Session([('arr1', ndtest(2)), ('arr2', ndtest((2, 2)))])
+        >>> s1.array_equals(s2)
+        name  arr1  arr2
+              True  True
+
+        Different value(s)
+
+        >>> s2.arr1['a1'] = 0
+        >>> s1.array_equals(s2)
+        name   arr1  arr2
+              False  True
+
+        Different label(s)
+
+        >>> s2.arr2 = ndtest("b=b0,b1; a=a0,a1")
+        >>> s1.array_equals(s2)
+        name   arr1   arr2
+              False  False
+
+        Extra/missing array(s)
+
+        >>> s2.arr3 = ndtest((3, 3))
+        >>> s1.array_equals(s2)
+        name   arr1   arr2   arr3
+              False  False  False
+        """
         self_keys = set(self.keys())
         all_keys = list(self.keys()) + [n for n in other.keys() if n not in self_keys]
+        def larray_nan_equal(a1, a2):
+            try:
+                a1 = aslarray(a1)
+            except Exception:
+                return False
+            return a1.equals(a2, nan_equals=True)
         res = [larray_nan_equal(self.get(key), other.get(key)) for key in all_keys]
         return LArray(res, [Axis(all_keys, 'name')])
 
-    def __ne__(self, other):
-        return ~(self == other)
+    def equals(self, other):
+        """Test if all arrays of the current session are equal to those of another session.
+
+        Parameters
+        ----------
+        other : Session
+            Session to compare with.
+
+        Returns
+        -------
+        True if arrays of both sessions are all equal, False otherwise.
+
+        See Also
+        --------
+        Session.array_equals
+
+        Examples
+        --------
+        >>> s1 = Session([('arr1', ndtest(2)), ('arr2', ndtest((2, 2)))])
+        >>> s2 = Session([('arr1', ndtest(2)), ('arr2', ndtest((2, 2)))])
+        >>> s1.equals(s2)
+        True
+
+        Different value(s)
+
+        >>> s2.arr1['a1'] = 0
+        >>> s1.equals(s2)
+        False
+
+        Different label(s)
+
+        >>> s2 = s1.copy()
+        >>> s2.arr2 = ndtest("b=b0,b1; a=a0,a1")
+        >>> s1.equals(s2)
+        False
+
+        Extra/missing array(s)
+
+        >>> s2 = s1.copy()
+        >>> s2.arr3 = ndtest((3, 3))
+        >>> s1.equals(s2)
+        False
+        """
+        return all(self.array_equals(other))
 
     def transpose(self, *args):
         """Reorder axes of arrays in session, ignoring missing axes for each array.
@@ -905,7 +1010,11 @@ class Session(object):
         return '\n'.join(template.format(**kwargs) for kwargs in templ_kwargs)
 
 
-def local_arrays(depth=0):
+def _exclude_private_vars(vars_dict):
+    return {k: v for k, v in vars_dict.items() if not k.startswith('_')}
+
+
+def local_arrays(depth=0, include_private=False):
     """
     Returns a session containing all local arrays sorted in alphabetical order.
 
@@ -913,6 +1022,8 @@ def local_arrays(depth=0):
     ----------
     depth: int
         depth of call frame to inspect. 0 is where `local_arrays` was called, 1 the caller of `local_arrays`, etc.
+    include_private: boolean, optional
+        Whether or not to include private local arrays (i.e. arrays starting with `_`). Defaults to False.
 
     Returns
     -------
@@ -920,10 +1031,12 @@ def local_arrays(depth=0):
     """
     # noinspection PyProtectedMember
     d = sys._getframe(depth + 1).f_locals
+    if not include_private:
+        d = _exclude_private_vars(d)
     return Session((k, d[k]) for k in sorted(d.keys()) if isinstance(d[k], LArray))
 
 
-def global_arrays(depth=0):
+def global_arrays(depth=0, include_private=False):
     """
     Returns a session containing all global arrays sorted in alphabetical order.
 
@@ -931,6 +1044,8 @@ def global_arrays(depth=0):
     ----------
     depth: int
         depth of call frame to inspect. 0 is where `global_arrays` was called, 1 the caller of `global_arrays`, etc.
+    include_private: boolean, optional
+        Whether or not to include private globals arrays (i.e. arrays starting with `_`). Defaults to False.
 
     Returns
     -------
@@ -938,10 +1053,12 @@ def global_arrays(depth=0):
     """
     # noinspection PyProtectedMember
     d = sys._getframe(depth + 1).f_globals
+    if not include_private:
+        d = _exclude_private_vars(d)
     return Session((k, d[k]) for k in sorted(d.keys()) if isinstance(d[k], LArray))
 
 
-def arrays(depth=0):
+def arrays(depth=0, include_private=False):
     """
     Returns a session containing all available arrays (whether they are defined in local or global variables) sorted in
     alphabetical order. Local arrays take precedence over global ones (if a name corresponds to both a local
@@ -951,6 +1068,8 @@ def arrays(depth=0):
     ----------
     depth: int
         depth of call frame to inspect. 0 is where `arrays` was called, 1 the caller of `arrays`, etc.
+    include_private: boolean, optional
+        Whether or not to include private arrays (i.e. arrays starting with `_`). Defaults to False.
 
     Returns
     -------
@@ -961,6 +1080,10 @@ def arrays(depth=0):
     global_vars = caller_frame.f_globals
     local_vars = caller_frame.f_locals
 
+    if not include_private:
+        global_vars = _exclude_private_vars(global_vars)
+        local_vars = _exclude_private_vars(local_vars)
+
     # We must first get all variables *then* filter by type, otherwise we could return a global array which is not
     # currently available because it is shadowed by a local non-array variable.
     all_keys = sorted(set(global_vars.keys()) | set(local_vars.keys()))
