@@ -1,5 +1,5 @@
 from __future__ import absolute_import, division, print_function
 
-from larray.inout.excel import open_excel
+from larray.inout.excel import *
 from larray.inout.array import *
 from larray.inout.session import *
