@@ -1,5 +1,6 @@
 from __future__ import absolute_import, print_function
 
+import os
 import csv
 import numpy as np
 import pandas as pd
@@ -9,7 +10,8 @@ from itertools import product
 from larray.core.axis import Axis
 from larray.core.array import LArray, ndtest
 from larray.core.group import _translate_sheet_name, _translate_key_hdf
-from larray.util.misc import basestring, skip_comment_cells, strip_rows, csv_open, StringIO, decode, unique
+from larray.util.misc import (basestring, skip_comment_cells, strip_rows, csv_open, StringIO, decode, unique,
+                              deprecate_kwarg)
 
 try:
     import xlwings as xw
@@ -184,7 +186,7 @@ def from_frame(df, sort_rows=False, sort_columns=False, parse_header=False, unfo
     return LArray(data, axes)
 
 
-def df_aslarray(df, sort_rows=False, sort_columns=False, raw=False, parse_header=True, **kwargs):
+def df_aslarray(df, sort_rows=False, sort_columns=False, raw=False, parse_header=True, wide=True, **kwargs):
     """
     Prepare Pandas DataFrame and then convert it into LArray.
 
@@ -203,6 +205,10 @@ def df_aslarray(df, sort_rows=False, sort_columns=False, raw=False, parse_header
     parse_header : bool, optional
         Whether or not to parse columns labels. Pandas treats column labels as strings.
         If True, column labels are converted into int, float or boolean when possible. Defaults to True.
+    wide : bool, optional
+        Whether or not to assume the array is stored in "wide" format.
+        If False, the array is assumed to be stored in "narrow" format: one column per axis plus one value column.
+        Defaults to True.
 
     Returns
     -------
@@ -216,19 +222,37 @@ def df_aslarray(df, sort_rows=False, sort_columns=False, raw=False, parse_header
     # irrespective of the actual data dimensionality
     if raw:
         columns = df.columns.values.tolist()
-        try:
-            # take the first column which contains '\'
-            pos_last = next(i for i, v in enumerate(columns) if isinstance(v, basestring) and '\\' in v)
-        except StopIteration:
-            # we assume first column will not contain data
-            pos_last = 0
+        if wide:
+            try:
+                # take the first column which contains '\'
+                pos_last = next(i for i, v in enumerate(columns) if isinstance(v, basestring) and '\\' in v)
+            except StopIteration:
+                # we assume first column will not contain data
+                pos_last = 0
 
-        # This is required to handle int column names (otherwise we can simply use column positions in set_index).
-        # This is NOT the same as df.columns[list(range(...))] !
-        index_columns = [df.columns[i] for i in range(pos_last + 1)]
-        # TODO: we should pass a flag to df_aslarray so that we can use inplace=True here
-        # df.set_index(index_columns, inplace=True)
-        df = df.set_index(index_columns)
+            # This is required to handle int column names (otherwise we can simply use column positions in set_index).
+            # This is NOT the same as df.columns[list(range(...))] !
+            index_columns = [df.columns[i] for i in range(pos_last + 1)]
+            df.set_index(index_columns, inplace=True)
+        else:
+            index_columns = [df.columns[i] for i in range(len(df.columns) - 1)]
+            df.set_index(index_columns, inplace=True)
+            series = df[df.columns[-1]]
+            if isinstance(series.index, pd.core.index.MultiIndex):
+                fill_value = kwargs.get('fill_value', np.nan)
+                # TODO: use argument sort=False when it will be available
+                # (see https://github.com/pandas-dev/pandas/issues/15105)
+                df = series.unstack(level=-1, fill_value=fill_value)
+                # pandas (un)stack and pivot(_table) methods return a Dataframe/Series with sorted index and columns
+                labels = df_labels(series, sort=False)
+                index = pd.MultiIndex.from_tuples(list(product(*labels[:-1])), names=series.index.names[:-1])
+                columns = labels[-1]
+                df = df.reindex(index=index, columns=columns, fill_value=fill_value)
+            else:
+                series.name = series.index.name
+                if sort_rows:
+                    raise ValueError('sort_rows=True is not valid for 1D arrays. Please use sort_columns instead.')
+                return from_series(series, sort_rows=sort_columns)
 
     # handle 1D
     if len(df) == 1 and (pd.isnull(df.index.values[0]) or
@@ -247,8 +271,24 @@ def df_aslarray(df, sort_rows=False, sort_columns=False, raw=False, parse_header
                           unfold_last_axis_name=unfold_last_axis_name, **kwargs)
 
 
-def read_csv(filepath_or_buffer, nb_index=None, index_col=None, sep=',', headersep=None, fill_value=np.nan,
-             na=np.nan, sort_rows=False, sort_columns=False, dialect='larray', **kwargs):
+def _get_index_col(nb_axes=None, index_col=None, wide=True):
+    if not wide:
+        if nb_axes is not None or index_col is not None:
+            raise ValueError("`nb_axes` or `index_col` argument cannot be used when `wide` argument is False")
+
+    if nb_axes is not None and index_col is not None:
+        raise ValueError("cannot specify both `nb_axes` and `index_col`")
+    elif nb_axes is not None:
+        index_col = list(range(nb_axes - 1))
+    elif isinstance(index_col, int):
+        index_col = [index_col]
+
+    return index_col
+
+
+@deprecate_kwarg('nb_index', 'nb_axes', arg_converter=lambda x: x + 1)
+def read_csv(filepath_or_buffer, nb_axes=None, index_col=None, sep=',', headersep=None, fill_value=np.nan,
+             na=np.nan, sort_rows=False, sort_columns=False, wide=True, dialect='larray', **kwargs):
     """
     Reads csv file and returns an array with the contents.
 
@@ -266,10 +306,13 @@ def read_csv(filepath_or_buffer, nb_index=None, index_col=None, sep=',', headers
     ----------
     filepath_or_buffer : str or any file-like object
         Path where the csv file has to be read or a file handle.
-    nb_index : int, optional
-        Number of leading index columns (ex. 4).
+    nb_axes : int, optional
+        Number of axes of output array. The first `nb_axes` - 1 columns and the header of the CSV file will be used
+        to set the axes of the output array. If not specified, the number of axes is given by the position of the
+        column header including the character `\` plus one. If no column header includes the character `\`, the array
+        is assumed to have one axis. Defaults to None.
     index_col : list, optional
-        List of columns for the index (ex. [0, 1, 2, 3]).
+        Positions of columns for the n-1 first axes (ex. [0, 1, 2, 3]). Defaults to None (see nb_axes above).
     sep : str, optional
         Separator.
     headersep : str or None, optional
@@ -282,6 +325,10 @@ def read_csv(filepath_or_buffer, nb_index=None, index_col=None, sep=',', headers
     sort_columns : bool, optional
         Whether or not to sort the columns alphabetically (sorting is more efficient than not sorting).
         Defaults to False.
+    wide : bool, optional
+        Whether or not to assume the array is stored in "wide" format.
+        If False, the array is assumed to be stored in "narrow" format: one column per axis plus one value column.
+        Defaults to True.
     dialect : 'classic' | 'larray' | 'liam2', optional
         Name of dialect. Defaults to 'larray'.
     **kwargs
@@ -292,23 +339,55 @@ def read_csv(filepath_or_buffer, nb_index=None, index_col=None, sep=',', headers
 
     Examples
     --------
-    >>> from larray.tests.common import abspath
-    >>> from larray import ndrange
-    >>> fpath = abspath('test.csv')
-    >>> a = ndrange('nat=BE,FO;sex=M,F')
-
-    >>> a.to_csv(fpath)
-    >>> read_csv(fpath)
+    >>> tmpdir = getfixture('tmpdir')
+    >>> fname = os.path.join(tmpdir.strpath, 'test.csv')
+    >>> a = ndtest('nat=BE,FO;sex=M,F')
+    >>> a
     nat\\sex  M  F
          BE  0  1
          FO  2  3
-    >>> read_csv(fpath, sort_columns=True)
+    >>> a.to_csv(fname)
+    >>> with open(fname) as f:
+    ...     print(f.read().strip())
+    nat\\sex,M,F
+    BE,0,1
+    FO,2,3
+    >>> read_csv(fname)
+    nat\\sex  M  F
+         BE  0  1
+         FO  2  3
+
+    Sort columns
+
+    >>> read_csv(fname, sort_columns=True)
     nat\\sex  F  M
          BE  1  0
          FO  3  2
-    >>> fpath = abspath('no_axis_name.csv')
-    >>> a.to_csv(fpath, dialect='classic')
-    >>> read_csv(fpath, nb_index=1)
+
+    Read array saved in "narrow" format (wide=False)
+
+    >>> a.to_csv(fname, wide=False)
+    >>> with open(fname) as f:
+    ...     print(f.read().strip())
+    nat,sex,value
+    BE,M,0
+    BE,F,1
+    FO,M,2
+    FO,F,3
+    >>> read_csv(fname, wide=False)
+    nat\\sex  M  F
+         BE  0  1
+         FO  2  3
+
+    Specify the number of axes of the output array (useful when the name of the last axis is implicit)
+
+    >>> a.to_csv(fname, dialect='classic')
+    >>> with open(fname) as f:
+    ...     print(f.read().strip())
+    nat,M,F
+    BE,0,1
+    FO,2,3
+    >>> read_csv(fname, nb_axes=2)
     nat\\{1}  M  F
          BE  0  1
          FO  2  3
@@ -327,20 +406,15 @@ def read_csv(filepath_or_buffer, nb_index=None, index_col=None, sep=',', headers
             line_stream = skip_comment_cells(strip_rows(reader))
             axes_names = next(line_stream)
 
-        if nb_index is not None or index_col is not None:
-            raise ValueError("nb_index and index_col are not compatible with dialect='liam2'")
+        if nb_axes is not None or index_col is not None:
+            raise ValueError("nb_axes and index_col are not compatible with dialect='liam2'")
         if len(axes_names) > 1:
-            nb_index = len(axes_names) - 1
+            nb_axes = len(axes_names)
         # use the second data line for column headers (excludes comments and blank lines before counting)
         kwargs['header'] = 1
         kwargs['comment'] = '#'
 
-    if nb_index is not None and index_col is not None:
-        raise ValueError("cannot specify both nb_index and index_col")
-    elif nb_index is not None:
-        index_col = list(range(nb_index))
-    elif isinstance(index_col, int):
-        index_col = [index_col]
+    index_col = _get_index_col(nb_axes, index_col, wide)
 
     if headersep is not None:
         if index_col is None:
@@ -363,7 +437,7 @@ def read_csv(filepath_or_buffer, nb_index=None, index_col=None, sep=',', headers
         df.index.names = combined_axes_names.split(headersep)
         raw = False
 
-    return df_aslarray(df, sort_rows=sort_rows, sort_columns=sort_columns, fill_value=fill_value, raw=raw)
+    return df_aslarray(df, sort_rows=sort_rows, sort_columns=sort_columns, fill_value=fill_value, raw=raw, wide=wide)
 
 
 def read_tsv(filepath_or_buffer, **kwargs):
@@ -421,8 +495,10 @@ def read_hdf(filepath_or_buffer, key, fill_value=np.nan, na=np.nan, sort_rows=Fa
     return df_aslarray(df, sort_rows=sort_rows, sort_columns=sort_columns, fill_value=fill_value, parse_header=False)
 
 
-def read_excel(filepath, sheetname=0, nb_index=None, index_col=None, fill_value=np.nan, na=np.nan,
-               sort_rows=False, sort_columns=False, engine=None, **kwargs):
+@deprecate_kwarg('nb_index', 'nb_axes', arg_converter=lambda x: x + 1)
+@deprecate_kwarg('sheetname', 'sheet')
+def read_excel(filepath, sheet=0, nb_axes=None, index_col=None, fill_value=np.nan, na=np.nan,
+               sort_rows=False, sort_columns=False, wide=True, engine=None, **kwargs):
     """
     Reads excel file from sheet name and returns an LArray with the contents
 
@@ -430,13 +506,16 @@ def read_excel(filepath, sheetname=0, nb_index=None, index_col=None, fill_value=
     ----------
     filepath : str
         Path where the Excel file has to be read.
-    sheetname : str, Group or int, optional
+    sheet : str, Group or int, optional
         Name or index of the Excel sheet containing the array to be read.
         By default the array is read from the first sheet.
-    nb_index : int, optional
-        Number of leading index columns (ex. 4). Defaults to 1.
+    nb_axes : int, optional
+        Number of axes of output array. The first `nb_axes` - 1 columns and the header of the Excel sheet will be used
+        to set the axes of the output array. If not specified, the number of axes is given by the position of the
+        column header including the character `\` plus one. If no column header includes the character `\`, the array
+        is assumed to have one axis. Defaults to None.
     index_col : list, optional
-        List of columns for the index (ex. [0, 1, 2, 3]). Default to [0].
+        Positions of columns for the n-1 first axes (ex. [0, 1, 2, 3]). Defaults to None (see nb_axes above).
     fill_value : scalar or LArray, optional
         Value used to fill cells corresponding to label combinations which are not present in the input.
         Defaults to NaN.
@@ -445,6 +524,10 @@ def read_excel(filepath, sheetname=0, nb_index=None, index_col=None, fill_value=
     sort_columns : bool, optional
         Whether or not to sort the columns alphabetically (sorting is more efficient than not sorting).
         Defaults to False.
+    wide : bool, optional
+        Whether or not to assume the array is stored in "wide" format.
+        If False, the array is assumed to be stored in "narrow" format: one column per axis plus one value column.
+        Defaults to True.
     engine : {'xlrd', 'xlwings'}, optional
         Engine to use to read the Excel file. If None (default), it will use 'xlwings' by default if the module is
         installed and relies on Pandas default reader otherwise.
@@ -455,17 +538,12 @@ def read_excel(filepath, sheetname=0, nb_index=None, index_col=None, fill_value=
         warnings.warn("read_excel `na` argument has been renamed to `fill_value`. Please use that instead.",
                       FutureWarning, stacklevel=2)
 
-    sheetname = _translate_sheet_name(sheetname)
+    sheet = _translate_sheet_name(sheet)
 
     if engine is None:
         engine = 'xlwings' if xw is not None else None
 
-    if nb_index is not None and index_col is not None:
-        raise ValueError("cannot specify both nb_index and index_col")
-    elif nb_index is not None:
-        index_col = list(range(nb_index))
-    elif isinstance(index_col, int):
-        index_col = [index_col]
+    index_col = _get_index_col(nb_axes, index_col, wide)
 
     if engine == 'xlwings':
         if kwargs:
@@ -473,31 +551,32 @@ def read_excel(filepath, sheetname=0, nb_index=None, index_col=None, fill_value=
                             .format(list(kwargs.keys())[0]))
         from larray.inout.excel import open_excel
         with open_excel(filepath) as wb:
-            return wb[sheetname].load(index_col=index_col, fill_value=fill_value, sort_rows=sort_rows,
-                                      sort_columns=sort_columns)
+            return wb[sheet].load(index_col=index_col, fill_value=fill_value, sort_rows=sort_rows,
+                                      sort_columns=sort_columns, wide=wide)
     else:
-        df = pd.read_excel(filepath, sheetname, index_col=index_col, engine=engine, **kwargs)
+        df = pd.read_excel(filepath, sheet, index_col=index_col, engine=engine, **kwargs)
         return df_aslarray(df, sort_rows=sort_rows, sort_columns=sort_columns, raw=index_col is None,
-                           fill_value=fill_value)
+                           fill_value=fill_value, wide=wide)
 
 
-def read_sas(filepath, nb_index=None, index_col=None, fill_value=np.nan, na=np.nan, sort_rows=False, sort_columns=False,
+@deprecate_kwarg('nb_index', 'nb_axes', arg_converter=lambda x: x + 1)
+def read_sas(filepath, nb_axes=None, index_col=None, fill_value=np.nan, na=np.nan, sort_rows=False, sort_columns=False,
              **kwargs):
     """
     Reads sas file and returns an LArray with the contents
-        nb_index: number of leading index columns (e.g. 4)
+        nb_axes: number of axes of the output array
     or
-        index_col: list of columns for the index (e.g. [0, 1, 3])
+        index_col: Positions of columns for the n-1 first axes (ex. [0, 1, 2, 3])
     """
     if not np.isnan(na):
         fill_value = na
         warnings.warn("read_sas `na` argument has been renamed to `fill_value`. Please use that instead.",
                       FutureWarning, stacklevel=2)
 
-    if nb_index is not None and index_col is not None:
-        raise ValueError("cannot specify both nb_index and index_col")
-    elif nb_index is not None:
-        index_col = list(range(nb_index))
+    if nb_axes is not None and index_col is not None:
+        raise ValueError("cannot specify both nb_axes and index_col")
+    elif nb_axes is not None:
+        index_col = list(range(nb_axes - 1))
     elif isinstance(index_col, int):
         index_col = [index_col]
 
@@ -505,7 +584,8 @@ def read_sas(filepath, nb_index=None, index_col=None, fill_value=np.nan, na=np.n
     return df_aslarray(df, sort_rows=sort_rows, sort_columns=sort_columns, fill_value=fill_value)
 
 
-def from_lists(data, nb_index=None, index_col=None, fill_value=np.nan, sort_rows=False, sort_columns=False):
+@deprecate_kwarg('nb_index', 'nb_axes', arg_converter=lambda x: x + 1)
+def from_lists(data, nb_axes=None, index_col=None, fill_value=np.nan, sort_rows=False, sort_columns=False, wide=True):
     """
     initialize array from a list of lists (lines)
 
@@ -513,11 +593,13 @@ def from_lists(data, nb_index=None, index_col=None, fill_value=np.nan, sort_rows
     ----------
     data : sequence (tuple, list, ...)
         Input data. All data is supposed to already have the correct type (e.g. strings are not parsed).
-    nb_index : int, optional
-        Number of leading index columns (ex. 4). Defaults to None, in which case it guesses the number of index columns
-        by using the position of the first '\' in the first line.
+    nb_axes : int, optional
+        Number of axes of output array. The first `nb_axes` - 1 columns and the header will be used
+        to set the axes of the output array. If not specified, the number of axes is given by the position of the
+        column header including the character `\` plus one. If no column header includes the character `\`, the array
+        is assumed to have one axis. Defaults to None.
     index_col : list, optional
-        List of columns for the index (ex. [0, 1, 2, 3]). Defaults to None (see nb_index above).
+        Positions of columns for the n-1 first axes (ex. [0, 1, 2, 3]). Defaults to None (see nb_axes above).
     fill_value : scalar or LArray, optional
         Value used to fill cells corresponding to label combinations which are not present in the input.
         Defaults to NaN.
@@ -526,6 +608,10 @@ def from_lists(data, nb_index=None, index_col=None, fill_value=np.nan, sort_rows
     sort_columns : bool, optional
         Whether or not to sort the columns alphabetically (sorting is more efficient than not sorting).
         Defaults to False.
+    wide : bool, optional
+        Whether or not to assume the array is stored in "wide" format.
+        If False, the array is assumed to be stored in "narrow" format: one column per axis plus one value column.
+        Defaults to True.
 
     Returns
     -------
@@ -543,6 +629,9 @@ def from_lists(data, nb_index=None, index_col=None, fill_value=np.nan, sort_rows
     sex\\year  1991  1992  1993
            M     0     1     2
            F     3     4     5
+
+    Read array with missing values + `fill_value` argument
+
     >>> from_lists([['sex', 'nat\\\\year', 1991, 1992, 1993],
     ...             [  'M', 'BE',           1,    0,    0],
     ...             [  'M', 'FO',           2,    0,    0],
@@ -552,15 +641,7 @@ def from_lists(data, nb_index=None, index_col=None, fill_value=np.nan, sort_rows
       M        FO   2.0   0.0   0.0
       F        BE   0.0   0.0   1.0
       F        FO   nan   nan   nan
-    >>> from_lists([['sex', 'nat', 1991, 1992, 1993],
-    ...             [  'M', 'BE',     1,    0,    0],
-    ...             [  'M', 'FO',     2,    0,    0],
-    ...             [  'F', 'BE',     0,    0,    1]], nb_index=2)
-    sex  nat\\{2}  1991  1992  1993
-      M       BE   1.0   0.0   0.0
-      M       FO   2.0   0.0   0.0
-      F       BE   0.0   0.0   1.0
-      F       FO   nan   nan   nan
+
     >>> from_lists([['sex', 'nat\\\\year', 1991, 1992, 1993],
     ...             [  'M', 'BE',           1,    0,    0],
     ...             [  'M', 'FO',           2,    0,    0],
@@ -570,36 +651,68 @@ def from_lists(data, nb_index=None, index_col=None, fill_value=np.nan, sort_rows
       M        FO     2     0     0
       F        BE     0     0     1
       F        FO    42    42    42
+
+    Specify the number of axes of the array to be read
+
+    >>> from_lists([['sex', 'nat', 1991, 1992, 1993],
+    ...             [  'M', 'BE',     1,    0,    0],
+    ...             [  'M', 'FO',     2,    0,    0],
+    ...             [  'F', 'BE',     0,    0,    1]], nb_axes=3)
+    sex  nat\\{2}  1991  1992  1993
+      M       BE   1.0   0.0   0.0
+      M       FO   2.0   0.0   0.0
+      F       BE   0.0   0.0   1.0
+      F       FO   nan   nan   nan
+
+    Read array saved in "narrow" format (wide=False)
+
+    >>> from_lists([['sex', 'nat', 'year', 'value'],
+    ...             [  'M', 'BE',  1991,    1     ],
+    ...             [  'M', 'BE',  1992,    0     ],
+    ...             [  'M', 'BE',  1993,    0     ],
+    ...             [  'M', 'FO',  1991,    2     ],
+    ...             [  'M', 'FO',  1992,    0     ],
+    ...             [  'M', 'FO',  1993,    0     ],
+    ...             [  'F', 'BE',  1991,    0     ],
+    ...             [  'F', 'BE',  1992,    0     ],
+    ...             [  'F', 'BE',  1993,    1     ]], wide=False)
+    sex  nat\\year  1991  1992  1993
+      M        BE   1.0   0.0   0.0
+      M        FO   2.0   0.0   0.0
+      F        BE   0.0   0.0   1.0
+      F        FO   nan   nan   nan
     """
-    if nb_index is not None and index_col is not None:
-        raise ValueError("cannot specify both nb_index and index_col")
-    elif nb_index is not None:
-        index_col = list(range(nb_index))
-    elif isinstance(index_col, int):
-        index_col = [index_col]
+    index_col = _get_index_col(nb_axes, index_col, wide)
 
     df = pd.DataFrame(data[1:], columns=data[0])
     if index_col is not None:
         df.set_index([df.columns[c] for c in index_col], inplace=True)
 
     return df_aslarray(df, raw=index_col is None, parse_header=False, sort_rows=sort_rows, sort_columns=sort_columns,
-                       fill_value=fill_value)
+                       fill_value=fill_value, wide=wide)
 
 
-def from_string(s, nb_index=None, index_col=None, sep=' ', **kwargs):
+@deprecate_kwarg('nb_index', 'nb_axes', arg_converter=lambda x: x + 1)
+def from_string(s, nb_axes=None, index_col=None, sep=' ', wide=True, **kwargs):
     """Create an array from a multi-line string.
 
     Parameters
     ----------
     s : str
         input string.
-    nb_index : int, optional
-        Number of leading index columns (ex. 4). Defaults to None, in which case it guesses the number of index columns
-        by using the position of the first '\' in the first line.
+    nb_axes : int, optional
+        Number of axes of output array. The first `nb_axes` - 1 columns and the header will be used
+        to set the axes of the output array. If not specified, the number of axes is given by the position of the
+        column header including the character `\` plus one. If no column header includes the character `\`, the array
+        is assumed to have one axis. Defaults to None.
     index_col : list, optional
-        List of columns for the index (ex. [0, 1, 2, 3]). Defaults to None (see nb_index above).
+        Positions of columns for the n-1 first axes (ex. [0, 1, 2, 3]). Defaults to None (see nb_axes above).
     sep : str
         delimiter used to split each line into cells.
+    wide : bool, optional
+        Whether or not to assume the array is stored in "wide" format.
+        If False, the array is assumed to be stored in "narrow" format: one column per axis plus one value column.
+        Defaults to True.
     \**kwargs
         See arguments of Pandas read_csv function.
 
@@ -653,4 +766,5 @@ def from_string(s, nb_index=None, index_col=None, sep=' ', **kwargs):
          BE  0  1
          FO  2  3
     """
-    return read_csv(StringIO(s), nb_index=nb_index, index_col=index_col, sep=sep, skipinitialspace=True, **kwargs)
+    return read_csv(StringIO(s), nb_axes=nb_axes, index_col=index_col, sep=sep, skipinitialspace=True,
+                    wide=wide, **kwargs)
