@@ -20,6 +20,10 @@ except ImportError:
     izip = zip
 
 import numpy as np
+try:
+    np.set_printoptions(legacy='1.13')
+except TypeError:
+    pass
 
 if sys.version_info[0] < 3:
     basestring = basestring
@@ -512,7 +516,30 @@ def float_error_handler_factory(stacklevel):
 
 
 def _isintstring(s):
-    return s.isdigit() or (len(s) > 1 and s[0] == '-' and s[1:].isdigit())
+    """
+    Return True if the passed string represents an integer.
+    Zero padded integers are considered as strings and not integers.
+
+    Parameters
+    ----------
+    s : str
+        string to test if representing an integer.
+
+    Examples
+    --------
+    >>> _isintstring('12')
+    True
+    >>> _isintstring('-12')
+    True
+    >>> _isintstring('a1')
+    False
+    >>> _isintstring('01')
+    False
+    """
+    def isposint(s):
+        # exclude zero padded strings
+        return s.isdigit() and not (len(s) > 1 and s[0] == '0')
+    return isposint(s) or (len(s) > 1 and s[0] == '-' and isposint(s[1:]))
 
 
 def _parse_bound(s, stack_depth=1, parse_int=True):
@@ -541,6 +568,8 @@ def _parse_bound(s, stack_depth=1, parse_int=True):
     2
     >>> _parse_bound('42')
     42
+    >>> _parse_bound('01')
+    '01'
     """
     s = s.strip()
     if s == '':
@@ -627,8 +656,8 @@ def renamed_to(newfunc, old_name, stacklevel=2):
     return wrapper
 
 # deprecate_kwarg is derived from pandas.util._decorators (0.21)
-def deprecate_kwarg(old_arg_name, new_arg_name, mapping=None, stacklevel=2):
-    if not isinstance(mapping, dict):
+def deprecate_kwarg(old_arg_name, new_arg_name, mapping=None, arg_converter=None, stacklevel=2):
+    if mapping is not None and not isinstance(mapping, dict):
         raise TypeError("mapping from old to new argument values must be dict!")
     def _deprecate_kwarg(func):
         @wraps(func)
@@ -637,13 +666,13 @@ def deprecate_kwarg(old_arg_name, new_arg_name, mapping=None, stacklevel=2):
             if old_arg_value is not None:
                 if mapping is not None:
                     new_arg_value = mapping.get(old_arg_value, old_arg_value)
-                    msg = "The {old_name}={old_val!r} keyword is deprecated, use {new_name}={new_val!r} instead"\
-                        .format(old_name=old_arg_name, old_val=old_arg_value, new_name=new_arg_name,
-                                new_val=new_arg_value)
+                elif arg_converter is not None:
+                    new_arg_value = arg_converter(old_arg_value)
                 else:
                     new_arg_value = old_arg_value
-                    msg = "The '{old_name}' keyword is deprecated, use '{new_name}' instead"\
-                        .format(old_name=old_arg_name, new_name=new_arg_name)
+                msg = "The {old_name}={old_val!r} keyword is deprecated, use {new_name}={new_val!r} instead"\
+                    .format(old_name=old_arg_name, old_val=old_arg_value, new_name=new_arg_name,
+                            new_val=new_arg_value)
 
                 warnings.warn(msg, FutureWarning, stacklevel=stacklevel)
                 if new_arg_name in kwargs:
